import warnings
from logging import getLogger
from nekbot.protocols import Message
from nekbot.protocols.xmpp.group_chat import GroupChatXMPP
from nekbot.protocols.xmpp.user import UserXMPP

__author__ = 'nekmo'

logger = getLogger('nekbot.protocols.xmpp.message')


class XMPPMessage(Message):
    def __init__(self, protocol, msg):
        user = UserXMPP(protocol, msg['mucnick'], msg['from'])
        if msg.xml.find('{urn:xmpp:delay}delay') is not None:
            self.historical = True
        self.msg = msg
        super(XMPPMessage, self).__init__(protocol, msg['body'], user)

    def get_group_chat_id(self):
        return self.msg['mucroom']

    def create_group_chat(self):
        GroupChatXMPP(self.protocol, self.msg['mucroom'].split('@')[0], self.msg['mucroom'])

    @property
    def is_public(self):
        return self.msg['type'] == 'groupchat'

    @property
    def is_own(self):
        return self.msg['from'].full == self.msg['to'].full

    @property
    def is_groupchat(self):
        return self.msg['mucroom'] is not None



from logging import getLogger
from nekbot.protocols.base.group_chat import GroupChats
from nekbot.protocols.xmpp.user import UsersXMPP

__author__ = 'nekmo'

from nekbot.protocols import GroupChat

logger = getLogger('nekbot.protocols.telegram.group_chat')


class GroupChatXMPP(GroupChat):

    def get_users(self, override=True):
        pass

    def send_message(self, body, notice=False):
        msubject = None
        mtype = 'groupchat'
        body = self.protocol.prepare_message(body)
        self.protocol.sleek_send_message(self.id, body, msubject, mtype)


class GroupChatsXMPP(GroupChats):
    pass


from sleekxmpp import JID
from nekbot.protocols.base.user import Users

__author__ = 'nekmo'

from nekbot.protocols import User


class UserXMPP(User):
    def __init__(self, protocol, username, user_id=None, groupchat=None):
        if not username:
            username = user_id.user
        if isinstance(user_id, JID):
            self.user = user_id
            user_id = user_id.bare
        else:
            self.user = None
        User.__init__(self, protocol, username, user_id)

    def send_message(self, body, notice=False):
        if self.user is None:
            raise ValueError('JID is not provided.')
        msubject = None
        mtype = 'chat'
        self.protocol.sleek_send_message(self.user, body, msubject, mtype)


class UsersXMPP(Users):
    pass


from collections import defaultdict
from logging import getLogger
import sleekxmpp
import sys
import time
from nekbot import settings
from nekbot.protocols.base import ProtocolBase
from nekbot.protocols.xmpp.message import XMPPMessage
from nekbot.protocols.xmpp.user import UserXMPP
from sleekxmpp import plugins
from xml.etree import cElementTree as ET


__author__ = 'nekmo'

logger = getLogger('nekbot.protocols.xmpp')

if sys.version_info < (3, 0):
    from sleekxmpp.util.misc_ops import setdefaultencoding

    setdefaultencoding('utf8')


class XMPP(sleekxmpp.ClientXMPP, ProtocolBase):
    features = ['newline', 'groupchats', 'history']
    user_class = UserXMPP
    name = 'XMPP'

    def __init__(self, nekbot):
        plugin_whitelist = plugins.__all__
        plugin_blacklist = settings.XMPP_PLUGIN_BLACKLIST
        for plugin in plugin_blacklist:
            plugin_whitelist.remove(plugin)
        sleekxmpp.ClientXMPP.__init__(self, settings.XMPP_JID, settings.XMPP_PASSWORD,
                                      plugin_config=self.get_plugin_config(),
                                      plugin_whitelist=plugin_whitelist, escape_quotes=True, sasl_mech=None, lang='en')
        ProtocolBase.__init__(self, nekbot)

    def get_plugin_config(self):
        # config = defaultdict(defaultdict)
        config = {}
        if True:
            config['xep_0048'] = {}
            config['xep_0048']['auto_join'] = True
        return config

    def prepare_message(self, body):
        if not isinstance(body, (str, unicode)):
            body = unicode(body)
        try:
            body = body.decode('utf-8')
        except:
            pass
        return body

    def sleek_send_message(self, mto, mbody, msubject=None, mtype=None,
                           mhtml=None, mfrom=None, mnick=None):
        return sleekxmpp.ClientXMPP.send_message(self, mto, mbody, msubject, mtype, mhtml, mfrom, mnick)

    def join_room(self, jid, nick=None, password=None):
        self.plugin['xep_0045'].joinMUC(jid, nick, password, wait=True)

    def join_rooms(self):
        for room in settings.XMPP_GROUPCHATS:
            self.join_room(**room)

    def start(self):
        self.register_plugins()
        self.add_event_handler("session_start", self.session_start)
        self.add_event_handler("message", self.message)
        self.connect()
        self.process()
        self.join_rooms()
        # time.sleep(40)

    def message(self, msg):
        self.propagate('message', XMPPMessage(self, msg))

    def session_start(self, session):
        self.send_presence()
        self.get_roster()

    def close(self):
        logger.debug('Closing SleekXMPP...')
        self.disconnect()


Xmpp = XMPP